(*
   minicrt version 1.0  Copyright (C) 2004 Frank Reglin

   minicrt is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   minicrt is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   See the GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with minicrt; see the file COPYING.
*)
unit minicrt;
(*
   subset of the crt unit
   F.Reglin 2004
*)

interface

const
 BW40 = 0;  (* video text modes *)
 CO40 = 1;
 BW80 = 2;
 CO80 = 3;
 Mono = 7;

var LastMode: word;

function  KeyPressed: boolean;
function  ReadKey: char;

procedure TextMode( mode: integer );
procedure ClrScr;
procedure GotoXY( x, y: integer );

procedure Delay( ms: word );

procedure Sound( hz: word );
procedure NoSound;


implementation

uses dos;

var
 bufchar: char;

function KeyPressed: boolean;
var
 reg: registers;
begin
   reg.ah := $1;
   intr( $16, reg ); (* BIOS Int 16H Fn 01h read keyboard status *)
   KeyPressed := (reg.flags and fzero) = 0;
end;

function ReadKey: char;
var
 reg: registers;
begin
   if bufchar <> chr(0)
   then begin
     ReadKey := bufchar;
     bufchar := chr(0);
   end
   else begin
      reg.ah := $0;
      intr( $16, reg ); (* BIOS Int 16H Fn 00h read keyboard input *)
      ReadKey := chr(reg.al);
      if reg.al = 0
      then begin
         bufchar := chr(reg.ah);
      end
   end
end;


procedure TextMode( mode: integer );
var
 reg: registers;
begin
   reg.ah := $0F;
   intr( $10, reg ); (* BIOS Int 10H Fn 0Fh return video status *)
   LastMode := reg.al;
   reg.ah := 0;
   reg.al := mode;
   intr( $10, reg ); (* BIOS Int 10H Fn 00h set video mode *)
   reg.ah := 5;
   reg.al := 0; (* page *)
   intr( $10, reg ); (* BIOS Int 10H Fn 05h select video page *)
end;

procedure ClrScr;
var
 reg: registers;
begin
   reg.ah := 6;
   reg.al := 0; (* clear *)
   reg.bh := 7; (* Attr *)
   reg.cx := 0;
   reg.dh :=24;
   reg.dl :=79;
   intr( $10, reg ); (* BIOS Int 10H Fn 06h scroll current page up *)
   GotoXY( 1, 1 );
end;

procedure GotoXY( x, y: integer );
var
 reg: registers;
begin
   reg.ah := 2;
   reg.bh := 0; (* page *)
   reg.dl := x - 1;
   reg.dh := y - 1;
   intr( $10, reg ); (* BIOS Int 10H Fn 05h set cursor position *)
end;


procedure Delay( ms: word );
var                               (* Aufloesung 55 msec (BIOS) *)
 h1, m1, s1, cs1, h2, m2, s2, cs2: word;
 time1, time2: longint;
begin
   gettime( h1, m1, s1, cs1 );
   time1 := (((longint(h1) * 60 + m1) * 60 + s1) * 100 + cs1) * 10;
   repeat
      gettime( h2, m2, s2, cs2 );
      time2 := (((longint(h2) * 60 + m2) * 60 + s2) * 100 + cs2) * 10;
   until (time2-time1) >= ms
end;


procedure Sound( hz: word );
(* siehe H.-P. Messmer, PC-Hardwarebuch, Addison-Wesley 1992,
   7.4 Lautsprecher, 7.4.1 PIT 8253/8254 *)
var
 counter: integer;
 hcnt, lcnt: byte;
begin
   counter := longint(1193180) div longint(hz);
   hcnt := hi(counter);
   lcnt := lo(counter);
   asm
      mov al , 10110110b
      out 43h, al
      mov al , lcnt
      out 42h, al
      mov al , hcnt
      out 42h, al
      in  al , 61h
      or  al , 00000011b
      out 61h, al
   end
end;

procedure NoSound;
begin
   asm
      in  al , 61h
      and al , 11111100b
      out 61h, al
   end
end;


begin (* init *)
   bufchar := chr(0);
end.

